@@ -1,5 +1,5 @@
 /* printf - format and print data
-   Copyright (C) 1990-2002, Free Software Foundation, Inc.
+   Copyright (C) 1990-2003, Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
    \t = horizontal tab
    \v = vertical tab
    \0ooo = octal number (ooo is 0 to 3 digits)
-   \xhhh = hexadecimal number (hhh is 1 to 3 digits)
+   \xhh = hexadecimal number (hhh is 1 to 2 digits)
    \uhhhh = 16-bit Unicode character (hhhh is 4 digits)
    \Uhhhhhhhh = 32-bit Unicode character (hhhhhhhh is 8 digits)
 
@@ -126,7 +126,7 @@ FORMAT controls the output as in C printf.  Interpreted sequences are:\n\
   \\v      vertical tab\n\
 "), stdout);
       fputs (_("\
-  \\xNNN   byte with hexadecimal value NNN (1 to 3 digits)\n\
+  \\xNN    byte with hexadecimal value NN (1 to 2 digits)\n\
 \n\
   \\uNNNN  character with hexadecimal value NNNN (4 digits)\n\
   \\UNNNNNNNN  character with hexadecimal value NNNNNNNN (8 digits)\n\
@@ -240,11 +240,11 @@ print_esc (const char *escstart)
   int esc_value = 0;		/* Value of \nnn escape. */
   int esc_length;		/* Length of \nnn escape. */
 
-  /* \0ooo and \xhhh escapes have maximum length of 3 chars. */
   if (*p == 'x')
     {
+      /* A hexadecimal \xhh escape sequence must have 1 or 2 hex. digits.  */
       for (esc_length = 0, ++p;
-	   esc_length < 3 && ISXDIGIT (*p);
+	   esc_length < 2 && ISXDIGIT (*p);
 	   ++esc_length, ++p)
 	esc_value = esc_value * 16 + hextobin (*p);
       if (esc_length == 0)
@@ -253,6 +253,8 @@ print_esc (const char *escstart)
     }
   else if (*p == '0')
     {
+      /* An octal \0ooo escape sequence has 0 to 3 octal digits
+	 after the leading \0.  */
       for (esc_length = 0, ++p;
 	   esc_length < 3 && isodigit (*p);
 	   ++esc_length, ++p)
@@ -281,7 +283,7 @@ print_esc (const char *escstart)
 	 0000009F, or 0000D800 through 0000DFFF inclusive. A universal
 	 character name shall not designate a character in the required
 	 character set.  */
-      if ((uni_value >= 0x00 && uni_value <= 0x9f
+      if ((uni_value <= 0x9f
 	   && uni_value != 0x24 && uni_value != 0x40 && uni_value != 0x60)
 	  || (uni_value >= 0xd800 && uni_value <= 0xdfff))
 	error (EXIT_FAILURE, 0, _("invalid universal character name \\%c%0*x"),
